import time
__author__ = 'zhoumx'

import pandas as pd
from sqlalchemy import create_engine

from  hotelindex.config import configs
from hotelindex.crawler.Wyn88Crawler import  Wyn88Crawler

class Utility:

    def __init__(self):
        self.mysqlurl = "mysql+pymysql://{}:{}@{}/{}?charset=utf8".format(
            configs["mysql"]["user"],
            configs["mysql"]["password"],
            configs["mysql"]["host"],
            configs["mysql"]["database"]
        )
        self.engine = create_engine(self.mysqlurl)
        self.cr = Wyn88Crawler()

    def WYN88RoomPrice_to_mysql(self,retryCount=10,sem=30,days=1,pause=10):
        for _ in range(retryCount):
            time.sleep(pause)
            try:
                roomPrices = self.cr.getRoomPrice(sem=sem,days=days)
            except Exception as e:
                print(e)
                sem = sem - 5
            else:
                df = pd.DataFrame(roomPrices)
                fetchTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
                df["FetchTime"] = fetchTime
                # 多值字段设置为字符串类型
                df['MemberDetails'] = df['MemberDetails'].astype(str)
                df['PromoDetails'] = df['PromoDetails'].astype(str)
                df.to_sql(configs["mysql"]["wyn88_room_price_tab_name"], self.engine, if_exists='append')
                # 更新酒店信息
                self.WYN88Hotel_to_mysql(self.hotelDetais)
                break

    def WYN88Hotel_to_mysql(self,hotels):
        df_hotels = pd.DataFrame(hotels)
        fetchTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        df_hotels["FetchTime"] = fetchTime
        # 多值字段设置为字符串类型
        df_hotels['Service'] = df_hotels['Service'].astype(str)
        df_hotels['ULogo'] = df_hotels['ULogo'].astype(str)
        df_hotels.to_sql(configs["mysql"]["wyn88_hotel_tab_name"], self.engine, if_exists='append')


if __name__ == "__main__":
    u =Utility()
    u.WYN88RoomPrice_to_mysql()


configs = {
    'mysql': {
        'host': '127.0.0.1',
        'port': 3306,
        'user': 'root',
        'password': 'root',
        'database': 'hotelindex',
        'wyn88_hotel_tab_name':'wyn88_hotel',
        'wyn88_room_price_tab_name': 'wyn88_room_price'
    }
}



__author__ = 'zhoumx'

import aiohttp
import asyncio
from lxml import etree
from urllib.request import urlopen, Request
import json
import datetime



class Wyn88Crawler:
    def __init__(self):
        self.session = aiohttp.ClientSession()
        self.cityCodes = {'1101': '北京市'} # {'1101': '北京市'}
        self.hotelCodes = {}
        self.hotelDetais = []
        self.roomPrices = []
        self.event_loop = asyncio.get_event_loop();
        # asyncio.BoundedSemaphore(),限制同时运行协程数量
        self.sem = asyncio.BoundedSemaphore(30)

    def initCityCodes(self):
        if len(self.cityCodes) == 0:
            request = Request("http://wap.wyn88.com/Home/City")
            lines = urlopen(request, timeout=60).read()
            linestr = lines.decode('utf-8');
            html = etree.HTML(linestr)
            result = html.xpath("//li/a")
            for el_a in result:
                if len(el_a.xpath("@cityno")) == 1:
                    self.cityCodes[el_a.xpath("@cityno")[0]] = el_a.xpath("text()")[0]
            print("initCityCodes:Done,count={}".format(len(self.cityCodes)))

    def readHotel(self, json_resp):
        self.hotelDetais += json.loads(json_resp.get("JsonData"))["Rows"]

    def readRoomPrices(self, json_resp):
        # pass
        hotelStatus = json.loads(json_resp.get("JsonData"))["hotelStatus"]
        hotelDetail = json.loads(json_resp.get("JsonData"))["hotelDetail"]
        for hs in hotelStatus:
            hs.update({"HotelName": hotelDetail["HotelName"]})
            hs.update({"HotelNo": hotelDetail["HotelNo"]})
        self.roomPrices += hotelStatus

    async def fetch(self, urlstr, callback=None):
        with(await self.sem):
            async with self.session.get(urlstr) as resp:
                assert resp.status == 200
                resp_json = await resp.json()
                callback(resp_json)

    def initHotels(self):
        self.initCityCodes()
        if len(self.hotelCodes) == 0:
            urlstr = "http://wap.wyn88.com/Hotel/GetHotelList?pageindex=1&pagesize=1000&cityno={0}"
            tasks = [self.fetch(urlstr.format(k), self.readHotel) for k in self.cityCodes]
            results = self.event_loop.run_until_complete(asyncio.gather(*tasks))
            for h in self.hotelDetais:
                self.hotelCodes[h["HotelNo"]] = h["HotelName"]
            print("initHotels:Done,count={}".format(len(self.hotelCodes)))

    def initRoomPrices(self, indate, days):
        self.initHotels()
        urlstr = "http://wap.wyn88.com/Hotel/GetHotelInfo?hotelno={0}&Indate={1}&Outdate={2}"
        tasks = []
        for d in range(days):
            df = indate + datetime.timedelta(days=d)
            dn = indate + datetime.timedelta(days=d + 1)
            tasks.extend([self.fetch(urlstr.format(k, df, dn), self.readRoomPrices) for k in self.hotelCodes])
        results = self.event_loop.run_until_complete(asyncio.gather(*tasks))
        print("initRoomPrices:Done,count={}".format(len(self.roomPrices)))


    def getRoomPrice(self, sem=None, indate=datetime.date.today(), days=1):
        if sem:
            self.sem = asyncio.BoundedSemaphore(sem)
        self.initRoomPrices(indate, days)
        return self.roomPrices





# coding=utf-8
from setuptools import setup, find_packages
setup(
      name='hotelindex',
      version='0.15',
      description="a crawler to get common hotel's room prices",
      keywords='crawler hotel plateno wyn88 ',
      author='zhoumx',
      author_email='franky.z.super@hotmail.com',
      url='https://github.com/zhoumx1987/hotelindex',
      classifiers={
        'Programming Language :: Python :: 3.6'
      },
      packages=['hotelindex','hotelindex.crawler'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[]
)